package com.couplet.business.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.couplet.analyze.common.contents.AnalyzeEventContents;
import com.couplet.analyze.common.event.AnalyzeEventCache;
import com.couplet.business.server.mapper.VehicleMapper;
import com.couplet.business.server.service.FenAndLogoService;
import com.couplet.business.server.service.VehicleAndLogoService;
import com.couplet.business.server.service.VehicleService;
import com.couplet.business.server.service.VehicleTypeService;
import com.couplet.common.core.domain.Result;
import com.couplet.common.core.utils.StringUtils;
import com.couplet.common.core.utils.uuid.UUID;
import com.couplet.common.domain.Fence;
import com.couplet.common.domain.Vehicle;
import com.couplet.common.domain.VehicleType;
import com.couplet.common.domain.request.VehicleEditParams;
import com.couplet.common.domain.request.VehicleInsertParams;
import com.couplet.common.domain.request.VehicleListParams;
import com.couplet.common.redis.service.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import java.util.HashSet;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @ProjectName: five-groups-couplet
 * @Author: LiuYunHu
 * @CreateTime: 2024/3/26
 * @Description: 车辆服务实现类
 */

@Service
@Slf4j
public class VehicleServiceImpl extends ServiceImpl<VehicleMapper, Vehicle> implements VehicleService {
    //车辆mapper
    @Autowired
    private VehicleMapper vehicleMapper;
    @Autowired
    private RedisService redis;

    //远程发送mq
//    @Autowired
//    private RemoteFenceService remoteFenceService;

    //车辆类型服务
    @Autowired
    private VehicleTypeService vehicleTypeService;

    //车辆与标志关联 的mapper
    @Autowired
    private VehicleAndLogoService vehicleAndLogoService;

    @Autowired
    private FenAndLogoService fenAndLogoService;

    @Autowired
    private AnalyzeEventCache eventCache;

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/26 22:11
     * @Description: 查询列表
     * @Param: [listParams]
     * @Return: java.util.List<com.couplet.vehicle.domain.LyhVehicle>
     **/
    @Override
    public List<Vehicle> list(VehicleListParams listParams) {
        // 创建查询条件包装器
        LambdaQueryWrapper<Vehicle> queryWrapper = new LambdaQueryWrapper<>();

        // 如果车辆类型不为空，添加车辆类型作为查询条件
        if (!StringUtils.isNull(listParams.getVehicleType())) {
            queryWrapper.eq(Vehicle::getVehicleType, listParams.getVehicleType());
        }

        // 如果车辆状态不为空，添加车辆状态作为查询条件
        if (!StringUtils.isNull(listParams.getVehicleState())) {
            queryWrapper.eq(Vehicle::getVehicleState, listParams.getVehicleState());
        }

        //只查询车辆未被删除的
        queryWrapper.eq(Vehicle::getIsdelete, 0);

        // 执行查询
        List<Vehicle> list = this.list(queryWrapper);

        //调用服务，得到车辆类型
        List<VehicleType> vehicleTypes = vehicleTypeService.list();

        //将类型id一样的记录 名称进行赋值
        list.forEach(vehicle -> {
            vehicleTypes.forEach(vehicleType -> {
                if (vehicle.getVehicleType() == vehicleType.getVehicleTypeId()) {
                    vehicle.setVehicleTypeName(vehicleType.getVehicleTypeName());
                }
            });
        });


        return list;
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/26 22:31
     * @Description: 通过id删除
     * @Param: [vehicleId]
     * @Return: java.lang.String
     **/
    @Override
    public String deleteById(Long vehicleId) {
        String result = "";

        UpdateWrapper<Vehicle> updateWrapper = new UpdateWrapper<>();

        updateWrapper.set("isdelete", 1)
                .eq("vehicle_id", vehicleId);

        boolean update = update(updateWrapper);

        if (!update) {
            result = "删除失败";
            Result.error(result);
        }

        //刷新set缓存
        reCache();

        result = "删除成功！";

        return result;
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/27 15:21
     * @Description: 通过id进行编辑
     * @Param: [editParams]
     * @Return: java.lang.String
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String editById(VehicleEditParams editParams) {
        String result = "";

        if ((editParams.getLogoIds() == null || editParams.getLogoIds().isEmpty())) {
            result = "未选择标识";
            Result.error(result);
        }

        UpdateWrapper<Vehicle> updateWrapper = new UpdateWrapper<>();

        //编辑车辆类型
        updateWrapper.set("vehicle_type", editParams.getVehicleType())

                //编辑电机厂商
                .set("motor_manufacturer", editParams.getMotorManufacturer())

                //编辑电池厂商
                .set("battery_manufacturer", editParams.getBatteryManufacturer())

                //编辑电机编号
                .set("motor_number", editParams.getMotorNumber())

                //编辑电池编号
                .set("battery_number", editParams.getBatteryNumber())

                //匹配车辆id
                .eq("vehicle_id", editParams.getVehicleId());


        boolean update = update(updateWrapper);

        if (!update) {
            result = "编辑失败";
            Result.error(result);
        }

        //删除掉车辆id为入参的车辆、标识的中间表数据
        int delete = vehicleAndLogoService.deleteByVehicleId(editParams.getVehicleId());

        //删除绑定之后，再添加新的绑定
        vehicleAndLogoService.vehicleBindLogo(editParams.getVehicleId(), editParams.getLogoIds());


        //刷新set缓存
        reCache();


        result = "编辑成功！";

        return result;
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/27 16:34
     * @Description: 新增车辆
     * @Param: [insertParams]
     * @Return: java.lang.String
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String insert(VehicleInsertParams insertParams) {
        String result = "";

        if ((insertParams.getLogoIds() == null || insertParams.getLogoIds().isEmpty())) {
            result = "未选择标识";
            Result.error(result);
        }


        //UUID生成17位随机字符串
        UUID uuid = UUID.randomUUID();
        String string = uuid.toString();
        String s = string.replaceAll("-", "");
        String vin = s.substring(0, 17);
//        System.out.println(vin+"   "+vin.length());

        /*
        //雪花算法生成随机数
        SnowflakeIdGenerator idGenerator = new SnowflakeIdGenerator(1, 1);
        long randomId = idGenerator.nextId();
        //转成字符串，便于切割
        String vin = "" + randomId;

        //切割，只留后17位
        vin = vin.substring(vin.length() - 17);

         */


        //创建入参对象
        Vehicle vehicle = new Vehicle();

        vehicle.setVehicleType(insertParams.getVehicleType());
        vehicle.setMotorManufacturer(insertParams.getMotorManufacturer());
        vehicle.setBatteryManufacturer(insertParams.getBatteryManufacturer());
        vehicle.setMotorNumber(insertParams.getMotorNumber());
        vehicle.setBatteryNumber(insertParams.getBatteryNumber());
        vehicle.setVin(vin);
        vehicle.setVehicleState(0);
        vehicle.setIsdelete(0);


        //执行插入操作
        int insert = vehicleMapper.insert(vehicle);

        if (insert == 0) {
            result = "新增失败";
            Result.error(result);
        }


        //获取新增的车辆id值
        //执行添加电子围栏
        int i = vehicleAndLogoService.vehicleBindLogo(vehicle.getVehicleId(), insertParams.getLogoIds());


        //刷新set缓存
        reCache();

        //加入事件缓存
        reEvent();


        result = "新增成功！";

        return result;
    }


    /*
     * @Author: LiuYunHu
     * @Date: 2024/3/31 21:34
     * @Description: 获取绑定当前车辆的标识
     * @Param: [vehicleId]
     * @Return: java.util.List<java.lang.Long>
     **/
    @Override
    public List<Long> getBindLogoById(Long vehicleId) {
        List<Long> logoIds = vehicleAndLogoService.getBindLogoById(vehicleId);

        return logoIds;
    }

    /*
     * @param userId:
     * @return List<Vehicle>
     * @author 付凡芮
     * @description 根据登入人id查询管理车辆
     *
     */
    @Override
    public List<Vehicle> UserUnderTheVehicleList(Long userId) {
        return vehicleMapper.UserUnderTheVehicleList(userId);
    }

    @Override
    public Integer deleteVehicle(Long middleId) {
        return vehicleMapper.deleteVehicle(middleId);
    }


    @Override
    public List<Vehicle> vehicleAll() {
        return vehicleMapper.vehicleAll();
    }


    /*
     * @Author: LiuYunHu
     * @Date: 2024/4/8 14:11
     * @Description: 通过vin修改车辆上下线的状态
     * @Param: [vin, status]
     * @Return: java.lang.Integer
     **/
    @Override
    public Integer onOrOutLineByVIN(String vin, int status) {

        return vehicleMapper.onOrOutLineByVIN(vin, status);
    }

    @Override
    public Integer addVehicle(Long userId, List<Long> vehicleIds) {
        return vehicleMapper.addVehicle(userId, vehicleIds);
    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/4/8 14:11
     * @Description: 通过vin查询车辆，因为是plus，所以是List
     * @Param: [vin]
     * @Return: java.util.List<com.couplet.common.domain.Vehicle>
     **/
    @Override
    public List<Vehicle> findByVIN(String vin) {

        //创建查询条件包装器
        LambdaQueryWrapper<Vehicle> queryWrapper = new LambdaQueryWrapper<>();

        if (!StringUtils.isEmpty(vin)) {
            queryWrapper.eq(Vehicle::getVin, vin);
        }

        //执行查询
        return this.list(queryWrapper);
    }


    /*
     * @Author: LiuYunHu
     * @Date: 2024/4/8 14:12
     * @Description: 定时执行，查询缓存中下线的车辆，修改其状态
     * @Param: []
     * @Return: void
     **/
    @Scheduled(cron = "0/1 * * * * *")
    public void downLine() {
        log.info("定时器启动");
        //先查询车辆列表
        List<Vehicle> list = this.list(new VehicleListParams(null, null, null, null));

        list.forEach(vehicle -> {
            try {
                //只针对已经上线的车辆
                if (redis.hasKey("已上线的车辆vin:" + vehicle.getVin())) {

                    //如果vin的缓存 时间还剩一秒，则判断为已经下线
                    if (redis.getExpire("已上线的车辆vin:" + vehicle.getVin()) <= 3) {
                        log.info(vehicle.getVin() + "的车辆已经下线");

                        //执行修改下线状态的方法
//                        Integer i = this.onOrOutLineByVIN(vehicle.getVin() + "," + 0);
                        Integer i = this.onOrOutLineByVIN(vehicle.getVin(), 0);

                        if (0 == i) {
                            log.error("下线状态修改失败");
                        } else {
                            log.info("下线状态修改成功");
                        }

                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        });

    }

    /*
     * @Author: LiuYunHu
     * @Date: 2024/4/8 14:14
     * @Description: 刷新缓存中 的数据      车辆绑定标识的缓存
     * @Param: []
     * @Return: void
     **/
//    @Scheduled(cron = "0/3 * * * * *")

    //初始化
    @PostConstruct
    public void reCache() {
        //刷新缓存执行开始

        //先获取所有车辆的信息
        List<Vehicle> list = list(new VehicleListParams());

        list.forEach(vehicle -> {
            //通过车辆的id获取对应的标识集合
            List<Long> bindLogoById = getBindLogoById(vehicle.getVehicleId());

            //遍历标识集合
            bindLogoById.forEach(logoId -> {

                //存标识id 的set
//                HashSet<Long> logos = new HashSet<>();
//                logos.add(logoId);
//                //存入redis
//                redis.setCacheSet("车辆vin和标识:" + vehicle.getVin(), logos);

//                redis.expire("车辆vin和标识:" + vehicle.getVin(), 5, TimeUnit.MINUTES);


                //通过标识id获取标识绑定的电子围栏集合
                List<Fence> fences = fenAndLogoService.findFencesByLogoId(logoId);
                //遍历电子围栏集合
                fences.forEach(fence -> {
                    HashSet<Fence> fanceSet = new HashSet<>();
                    fanceSet.add(fence);
                    //存入redis
                    redis.setCacheSet("车辆vin和电子围栏:" + vehicle.getVin(), fanceSet);
//                    redis.expire("车辆vin和电子围栏:" + vehicle.getVin(), 5, TimeUnit.MINUTES);
                });
            });
        });
    }


    /*
     * @Author: LiuYunHu
     * @Date: 2024/4/8 19:39
     * @Description: 初始化   车辆事件 默认车辆有所有事件
     * @Param: []
     * @Return: void
     **/
    @PostConstruct
    public void reEvent() {
        List<Vehicle> list = list(new VehicleListParams());

        list.forEach(vehicle -> {
            eventCache.addEvent(vehicle.getVin(), AnalyzeEventContents.STORED_EVENT);
            eventCache.addEvent(vehicle.getVin(), AnalyzeEventContents.BREAKDOWN);
            eventCache.addEvent(vehicle.getVin(), AnalyzeEventContents.ELECTRONIC_FENCE);
            eventCache.addEvent(vehicle.getVin(), AnalyzeEventContents.REAL_TIME_DATA);
        });
    }
}
